/* $Id: eeread.c,v 1.6 1996/03/26 22:49:14 james Exp $ */
/*
 * $Log: eeread.c,v $
 * Revision 1.6  1996/03/26 22:49:14  james
 * We now use library routines for decoding and encoding the config
 * byte.  We now decode also the DSP address in the comment for byte
 * zero.  To fit everything on one line the whitespace has been reduces
 * and the format of the text has changed.
 *
 *
 * Also, support setting the DSP/Mixer address.
 *
 * Revision 1.5  1996/03/25 02:19:07  james
 * Moved the names of rthe fields of the config bytes into jy_sg.h.
 *
 * Revision 1.4  1995/03/31 05:52:43  james
 * Removed the more verbose progress messages.
 *
 * Revision 1.3  1995/03/31  04:07:41  james
 * Changed format so that it's practical to have eewrite read in either a
 * binary or an ascii file.
 *
 * Added comments showing the meanings of the values we have read.
 *
 * Revision 1.2  1995/03/31  02:56:47  james
 * Changed the upper limit for EEPROM reading to match the card I
 * actually have.
 *
 * Revision 1.1  1995/03/30  20:25:14  james
 * Initial revision
 *
 */

/* Utility to read the Sound Galaxy NX Pro 16's EEPROM. */

#include <stdio.h>		/* perror() */
#include <string.h>		/* strdup() */
#include <assert.h>		/* assert() */
#include <limits.h>		/* CHAR_BIT */
#include <getopt.h>		/*  getopt_long() */

#include "jy_sg.h"

#define PORT_IO_DEVICE "/dev/port" /* We may need setgid kmem. */

#define ASSERT(x) assert(x)

#define INT_BITS ( ( CHAR_BIT * sizeof(int) ) / sizeof(char) )

void
print_in_binary(FILE* fp, unsigned width, char fill_char, unsigned int n)
{
  int pos;
  int filling;

  if (fill_char != '0' && fill_char != ' ')
    fill_char = ' ';

  while(width > INT_BITS)
    {
      fputc(fill_char, fp);
      width--;
    }
  
  filling = 1;
  for(pos=INT_BITS-1; pos>=0; pos--)
    {
      char digit;
      
      if (pos >= INT_BITS)
	{
	  digit = fill_char;
	}
      else
	{
	  if ( n & (1<<pos) )	/* Mask out the bits starting at top */
	    {
	      digit = '1';
	      filling = 0;
	    }
	  else
	    {
	      digit = '0';
	    }
	}

      if (pos >= width)
	{
	  /* We're checking a digit we dont */
	  /* want to print unless we have to. */
	  if (digit == '1')
	    {
	      fputc(digit, fp);

	      /* From now on, we can't avoid printing the zeroes. */
	      width = pos;
	      fill_char = '0';
	    }
	  else
	    {
	      /* Outside the width, digit is zero => do nothing */
	    }
	}
      else
	{
	  /* We're inside the width */
	  if (digit == '1')
	    {
	      fputc(digit, fp);
	      fill_char = '0';
	    }
	  else
	    {
	      /* We are inside the width, */
	      /* print a fill char */
	      /* fill_char becomes 0 when we print the leading 1. */
	      fputc(fill_char, fp);
	    }
	}	  
    }
}

static void
show_meaning(FILE *fp, BYTE location, BYTE val)
{
  const char *location_names[]=
    {
      "Configuration", "Voice Volume", "Microphone", "ADC Input control",
      "Output Setting", "Master Volume", "FM Volume", "CD Volume",
      "Line In Volume", "Covox Volume", "Treble", "Bass", "8/16-bit control"
    };
        
    if (location == 0)
      {
	struct sgnx_eeprom_config cfg;
	decode_sgnx_config_byte(val, &cfg);
	fprintf(fp, "%s", decode_sgnx_config(&cfg));
      }
    else if (location > 0 && location <= 0x0C)
      {
	fprintf(fp, location_names[location]);
      }
}

/*
 * static variables for command-line options.
 */
static int use_ascii;
static const char short_opts[] = "ab"; /* a for ASCII, b for Binary */
static struct option long_opts[] = 
{
  /* name    has_arg    int *flag,    val  */
  { "ascii",	0,	&use_ascii,	1 },
  { "binary",	0,	&use_ascii,	0 },

  { 0,		0,	0,		0 } /* end marker */
};
    
int
main(int argc, char *argv[])
{
  BYTE eeprom_location;
  BYTE eeprom_data[SG_NX_PRO_EEPROM_SIZE];
  
  /* Set the defaults */
  use_ascii =  1;

  while (1)
    {
      int optind;
      int c;

      c = getopt_long(argc, argv, short_opts, long_opts, &optind);
      if (c == -1)
	break;
      switch (c)		/* cope with short options */
	{
	case 'a':
	  use_ascii = 1;
	  break;
	case 'b':
	  use_ascii = 0;
	  break;
	}
    }      
  /* Done processing options. */
  fprintf(stderr, "%s output.\n", use_ascii ? "ASCII" : "Binary");
  
  fflush(stderr);
  if ( ! open_port_device(PORT_IO_DEVICE) )
    {
      fprintf(stderr, "Can\'t open Port I/O device, can\'t continue.\n");
      return 1;
    }
  
  /* Port IO device opened OK */

  /* Read the data. */
  memset(eeprom_data, 0, SG_NX_PRO_EEPROM_SIZE);
  read_eeprom_block(eeprom_data, SG_NX_PRO_EEPROM_SIZE);
  
  if (use_ascii)
    {
      printf("# Byte  Offset  Bits  \n");
      printf("#               76543210\n");
      /*              1         2         3 */
      /*     123456789012345678901234567890 */
    }

  for(eeprom_location=0; eeprom_location < SG_NX_PRO_EEPROM_SIZE; eeprom_location++)
    {
      BYTE val = eeprom_data[eeprom_location];
      
      if (use_ascii)
	{
	  printf("%02X    # %02X      ",
		 (unsigned int)val,
		 (unsigned int)eeprom_location);
	  
	  print_in_binary(stdout, 8, '0', val);
	  putchar(' ');
	  show_meaning(stdout, eeprom_location, val);
	  printf("\n");
	  fflush(stdout);
	}
      else
	{
	  fputc(val, stdout);
	}
    }
  close_port_device();
  
  return 0;
}

